import unittest
import os
from kojen.Install import *
from filecmp import dircmp

relative_root = "trash"
relative_paths = [
    "nested",
    "nested/nested",
    "nested/nested/nested",
]

class TestInstall(unittest.TestCase):
    def setUp(self):
        """Will create one per path."""
        self.trash_root = os.path.normpath(os.path.join(os.getcwd(), relative_root))
        self.trash_paths = []
        self.relative_file_paths = []
        self.filenames = []
        cnt = 1
        self.trash_paths.append(self.trash_root)
        fn = "file" + str(cnt) + ".py"
        self.relative_file_paths.append(os.path.normpath(fn))
        self.filenames.append(fn)
        cnt = cnt + 1
        for p in relative_paths:
            self.trash_paths.append(os.path.normpath(os.path.join(self.trash_root,p)))
            fn = "file" + str(cnt) + ".py"
            self.relative_file_paths.append(os.path.normpath(os.path.join(p, fn)))
            self.filenames.append(fn)
            cnt = cnt + 1
        cnt = 1
        for p in self.trash_paths:
            if not os.path.exists(p):
                os.makedirs(p)
            filename = os.path.normpath(os.path.join(p, "file" + str(cnt) + ".py"))
            with open(filename, "w") as file:
                file.write("#!/usr/bin/env python\n")
                file.write("import sys\n")
                file.write("sys.exit(0)")
            cnt = cnt + 1

    def tearDown(self):
        shutil.rmtree(self.trash_root)

    def checkDirCmpResult(self, result):
        """Recursive check of file differences in folders"""
        self.assertFalse(result.diff_files)
        for sub_dcmp in result.subdirs.values():
            self.checkDirCmpResult(sub_dcmp)

    def test_InstallTemplates(self):
        InstallTemplates(self.trash_root)
        self.checkDirCmpResult(dircmp(self.trash_root, getUserTemplateRoot()))

    def test_ContainsTemplates(self):
        InstallTemplates(self.trash_root)
        # Found cases
        for p in relative_paths:
            self.assertTrue(ContainsTemplates(p))
        for p in self.filenames:
            self.assertTrue(ContainsTemplates(p))
        for p in self.relative_file_paths:
            self.assertTrue(ContainsTemplates(p))
        # Not found cases
        for p in relative_paths:
            self.assertFalse(ContainsTemplates(p.replace("n","t")))
        for p in self.filenames:
            self.assertFalse(ContainsTemplates(p.replace("py","cs")))
        for p in self.relative_file_paths:
            self.assertFalse(ContainsTemplates(p.replace("py","cs")))
        self.assertFalse(ContainsTemplates("File6.py"))

    def test_UninstallTemplates(self):
        InstallTemplates(self.trash_root)
        UninstallTemplates()
        self.assertFalse(os.path.exists(getUserTemplateRoot()))




#!/usr/bin/env python3

'''
  @file
  @ingroup <<<GROUP>>>
  @brief   <<<BRIEF>>>

           This code is Autogenerated from '<<<PYIFGENNAME>>>' with the MIT License.
           As such, please only hand-code within 'USER' tags.

  @author  <<<AUTHOR>>>
'''

import unittest
import threading
import queue

from <<<STATEMACHINENAME>>>Controller import *
from <<<STATEMACHINENAME>>>StateMachine import *
# {{{USER_IMPORTS}}}
# {{{USER_IMPORTS}}}

class Test<<<STATEMACHINENAME>>>Controller(<<<STATEMACHINENAME>>>Controller):
    ## Test context constructor.
    def __init__(self):
        <<<STATEMACHINENAME>>>Controller.__init__(self)
        # {{{USER_CONSTRUCTOR}}}
        # {{{USER_CONSTRUCTOR}}}

    # Overridden guards.
    #
    <<<PER_GUARD_BEGIN>>>
    def <<<GUARDNAME>>>(self, event) -> bool:
        # {{{USER_<<<GUARDNAME>>>}}}
        # {{{USER_<<<GUARDNAME>>>}}}
        return <<<STATEMACHINENAME>>>Controller.<<<GUARDNAME>>>(self, event)
    <<<PER_GUARD_END>>>

    ## Overridden on state entry/exit.
    #
    <<<PER_STATE_BEGIN>>>
    def On<<<STATENAME>>>Entry(self, event) -> None:
        <<<STATEMACHINENAME>>>Controller.On<<<STATENAME>>>Entry(self, event)
        # {{{USER_<<<STATENAME>>>_on_entry}}}
        # {{{USER_<<<STATENAME>>>_on_entry}}}

    def On<<<STATENAME>>>Exit(self, event) -> None:
        <<<STATEMACHINENAME>>>Controller.On<<<STATENAME>>>Exit(self, event)
        # {{{USER_<<<STATENAME>>>_on_exit}}}
        # {{{USER_<<<STATENAME>>>_on_exit}}}
    <<<PER_STATE_END>>>

    ## Overridden actions.
    # </summary>
    <<<PER_ACTION_BEGIN>>>
    def <<<ACTIONNAME>>>(self, event) -> None:
        <<<STATEMACHINENAME>>>Controller.<<<ACTIONNAME>>>(self, event);
        # {{{USER_<<<ACTIONNAME>>>_<<<EVENTNAME>>>}}}
        # {{{USER_<<<ACTIONNAME>>>_<<<EVENTNAME>>>}}}
    <<<PER_ACTION_END>>>
    # {{{USER_MEMBERS}}}
    # {{{USER_MEMBERS}}}

## Python Unittest
class Test<<<STATEMACHINENAME>>>(unittest.TestCase):

    def setUp(self):
        self.context = Test<<<STATEMACHINENAME>>>Controller()
        self.sm = <<<STATEMACHINENAME>>>StateMachine(self.context)

    def test_states(self):
        # {{{USER_UNIT_TEST_STATES}}}
        # {{{USER_UNIT_TEST_STATES}}}
        pass

## Console Runner Exit
#
class Exit(Exception):
    pass

## Console Runner Test Suite
# Run a statemachine on the console taking keypresses as events.
#
class ConsoleRunner(threading.Thread):
    def __init__(self):
        threading.Thread.__init__(self)
        self.context = Test<<<STATEMACHINENAME>>>Controller()
        self.sm = <<<STATEMACHINENAME>>>StateMachine(self.context)
        self.__runThreaded = True
        self.__fifoQueue = queue.Queue()  # threadsafe by default.
        # {{{USER_CONSOLE_RUNNER}}}
        # {{{USER_CONSOLE_RUNNER}}}
        self.start()

    ## @brief Threading
    #
    # @{
    def run(self) -> None:
        while self.__runThreaded:
            try:
                event = self.__fifoQueue.get(block=True, timeout=5)  # 5s timeout for graceful shutdown.
                self.process(event)
                self.__fifoQueue.task_done()
            except queue.Empty:
                pass

    def stop(self) -> None:
        self.sm.stop()
        if self.__runThreaded:
            self.__runThreaded = False
            self.__fifoQueue.join()
            self.join()
    # @}

    def printHelp(self):
        print("##############################################")
        <<<PER_EVENT_BEGIN>>>
        print("<<<ALPH>>>      : send <<<EVENTNAME>>> event.")
        <<<PER_EVENT_END>>>
        <<<PER_GUARD_BEGIN>>>
        print('g<<<NUM>>>=0/1 : set <<<GUARDNAME>>> to false/true')
        <<<PER_GUARD_END>>>
        print("H      : print this HELP.")
        print("exit   : EXIT the console runner.")
        print("##############################################")

    def processConsoleInput(self, input):
        <<<PER_EVENT_BEGIN>>>
        if input == '<<<ALPH>>>':
            self.sm.Trigger<<<EVENTNAME>>>(<<<SIGNATUREWITHDEFAULTS>>>)
            return
        <<<PER_EVENT_END>>>
        <<<PER_GUARD_BEGIN>>>
        if 'g<<<NUM>>>=' in input:
            input = input.replace('g<<<NUM>>>=', '').strip()
            self.context.<<<guardName>>> = False if '0' in input else True
            print ("<<<guardName>>> set as ", "false" if not self.context.<<<guardName>>> else "true")
            return
        <<<PER_GUARD_END>>>
        if input == 'H':
            self.printHelp()
            return
        if input == 'exit':
            raise Exit
        print("'", input, '" not defined.')
    

    def startConsole(self):
        import sys
        self.printHelp()
        try:
            for line in sys.stdin:
                line = line.replace("\n", "")
                for var in line.split():
                    self.processConsoleInput(var)
        except Exit:
            print("Goodbye...")

    # {{{USER_TESTS}}}
    # {{{USER_TESTS}}}

if __name__ == '__main__':
    testrunner = ConsoleRunner()
    # {{{USER_MAIN}}}
    # {{{USER_MAIN}}}
    testrunner.startConsole()
    testrunner.stop()


#!/usr/bin/env python3

'''
  @file
  @ingroup <<<GROUP>>>
  @brief   <<<BRIEF>>>

           This code is Autogenerated from '<<<PYIFGENNAME>>>' with the MIT License.
           As such, please only hand-code within 'USER' tags.

  @author  <<<AUTHOR>>>
'''

from enum import Enum, unique, auto
# {{{USER_IMPORTS}}}
# {{{USER_IMPORTS}}}

## @briefEvents
#
#@{
<<<PER_EVENT_BEGIN>>>
class <<<EVENTNAME>>>:
    def __init__(self, <<<SIGNATURE>>>):
        <<<EVENTMEMBERSLITEINSTANTIATE=self>>>
        pass
<<<PER_EVENT_END>>>

class EventStartup:
    pass
#@}

## Controller Interface
#
class <<<STATEMACHINENAME>>>Controller:
    def __init__(self):
        <<<PER_GUARD_BEGIN>>>
        self.<<<guardName>>> = False
        <<<PER_GUARD_END>>>
        pass
    ## @brief Guards
    #
    # @{
    <<<PER_GUARD_BEGIN>>>
    def <<<GUARDNAME>>>(self, event) -> bool:
        print("<<<GUARDNAME>>> returned as ", str(self.<<<guardName>>>))
        return self.<<<guardName>>>
    <<<PER_GUARD_END>>>
    #@}

    ## @brief Actions
    #
    #@{
    <<<PER_ACTION_BEGIN>>>
    def <<<ACTIONNAME>>>(self, event) -> None:
        print("<<<ACTIONNAME>>> on ", str(type(event)).replace("<class '",'').replace("'>",''))
    <<<PER_ACTION_END>>>
    #@}

    ## @brief State Entry/Exit
    #
    #@{
    <<<PER_STATE_BEGIN>>>
    def On<<<STATENAME>>>Entry(self, event) -> None:
        print("On<<<STATENAME>>>Entry on ", str(type(event)).replace("<class '",'').replace("'>",''))
    def On<<<STATENAME>>>Exit(self, event) -> None:
        print("On<<<STATENAME>>>Exit(self, event) on ", str(type(event)).replace("<class '",'').replace("'>",''))
    <<<PER_STATE_END>>>
    #@}

    ## @brief Handler function for non-transition events.
    # @param Event ignored by state machine.
    #
    def NoTransition(self, event) -> None:
        pass




#!/usr/bin/env python3

'''
  @file
  @ingroup <<<GROUP>>>
  @brief   <<<BRIEF>>>

           This statemachine has the following transition table:

           <<<TTT_BOOST_SML>>>

           This code is Autogenerated from '<<<PYIFGENNAME>>>' with the MIT License.
           As such, please only hand-code within 'USER' tags.

  @author  <<<AUTHOR>>>
'''

from <<<STATEMACHINENAME>>>Controller import *
import threading
import queue
from enum import Enum, unique, auto
# {{{USER_IMPORTS}}}
# {{{USER_IMPORTS}}}

## State IDs
@unique
class <<<STATEMACHINENAME>>>StateId(Enum):
    <<<PER_STATE_BEGIN>>>
    c<<<STATENAME>>> = auto()
    <<<PER_STATE_END>>>

##  <<<STATEMACHINENAME>>>StateMachine Implementation
#
class <<<STATEMACHINENAME>>>StateMachine(threading.Thread):
    ## Constructor
    # @param controller context instance of the <<<STATEMACHINENAME>>>ControllerInterface
    def __init__(self, controller):
        threading.Thread.__init__(self)
        self.context        = controller
        self.__runThreaded  = <<<StateMachineThread=1>>>
        self.__fifoQueue    = queue.Queue() # threadsafe by default.
        self.context.On<<<STATE_0>>>Entry(EventStartup())
        self.currentState = <<<STATEMACHINENAME>>>StateId.c<<<STATE_0>>>
        if self.__runThreaded:
            self.start()

    ## @brief State check
    #
    #@{
    <<<PER_STATE_BEGIN>>>
    def Is<<<STATENAME>>>(self) -> bool:
        return self.currentState == <<<STATEMACHINENAME>>>StateId.c<<<STATENAME>>>
    <<<PER_STATE_END>>>
    #@}

    ## @brief Trigger events from other threads or interrupt service routines.
    #
    #@{
    <<<PER_EVENT_BEGIN>>>
    def Trigger<<<EVENTNAME>>>(self, <<<SIGNATURE>>>) -> None:
        event = <<<EVENTNAME>>>(<<<SIGNATURE>>>)
        if self.__runThreaded:
            self.__fifoQueue.put(event)
        else:
            self.process(event)
    <<<PER_EVENT_END>>>
    #@}

    ## @brief Threading
    #
    #@{
    def run(self) -> None:
        while self.__runThreaded:
            try:
                event = self.__fifoQueue.get(block=True, timeout=5) # 5s timeout for graceful shutdown.
                self.process(event)
                self.__fifoQueue.task_done()
            except queue.Empty:
                pass

    def stop(self) -> None:
        if self.__runThreaded:
            self.__runThreaded = False
            self.__fifoQueue.join()
            self.join()
    #@}

    ## @brief State Processing
    #
    #@{
    def process(self, event) -> None:
        #print("<<<STATEMACHINENAME>>>StateMachine::process ", str(type(event)).replace("<class '",'').replace("'>",''))
        <<<PER_STATETRANSITION_BEGIN>>>
        if self.currentState == <<<STATEMACHINENAME>>>StateId.c<<<STATENAME>>>:
            self.process<<<STATENAME>>>(event)
            return
        <<<PER_STATETRANSITION_END>>>

    <<<PER_STATETRANSITION_BEGIN>>>
    def process<<<STATENAME>>>(self, event) -> None:
        <<<PER_EVENTTRANSITION_BEGIN>>>
        if isinstance(event, <<<EVENTNAME>>>):
            <<<PER_GUARDTRANSITION_BEGIN>>>
            if self.context.<<<GUARDNAME>>>(event):
                self.context.On<<<STATENAMEIFNEXTSTATE>>>Exit(event)
                self.context.<<<ACTIONNAME>>>(event)
                self.context.On<<<NEXTSTATENAME>>>Entry(event)
                self.currentState = <<<STATEMACHINENAME>>>StateId.c<<<NEXTSTATENAME>>>
                return
            <<<PER_GUARDTRANSITION_END>>>
        <<<PER_EVENTTRANSITION_END>>>
        print("<<<STATEMACHINENAME>>>StateMachine no transition in state '<<<STATENAME>>>' on event ", str(type(event)).replace("<class '",'').replace("'>",''));
        self.context.NoTransition(event)

    <<<PER_STATETRANSITION_END>>>
    #@}
